<?php
/**
 * @author Denny Vrandecic
 *
 * This feature implements inline queries, i.e. it offers the possibility
 * to ask Mediawiki's own Semantic knowledgebase stuff and replaces these
 * questions with answers.
 *
 * I guess, with good knowledge of MySQL the created SQL could be speeded up
 * a lot. So, if you have the knowledge, feel free to make it better.
 */

require_once( "$IP/includes/Title.php" );

// Here are some globals the administrator may set in his local settings.

// Default linking behaviour. Can be one of "none", "head", "all"
if (!isset( $smwgDefaultLinking )) $smwgDefaultLinking = "head";
// Maximal number of rows returned in a query
if (!isset( $smwgMaximalLimit )) $smwgMaximalLimit = 1000;
// Default number of rows returned in a query
if (!isset( $smwgStandardLimit )) $smwgStandardLimit = 100;
// If true, inner queries are enabled. May cost performance
if (!isset( $smwgInnerQueriesEnabled )) $smwgInnerQueriesEnabled = TRUE;
// If true, disjunctive queries are enabled. May cost performance
if (!isset( $smwgDisjunctiveQueriesEnabled )) $smwgDisjunctiveQueriesEnabled = TRUE;
// Sets the level of inclusions by subCategory-relations. For example, if Student
// is a Subcategory of Person, Students will only be returned if querying for Persons
// if this is set at least to 1 (as it is a direct Subcategory).
// Disjunctive queries must be enabled for this feature to work. May cost performance
if (!isset( $smwgSubcategoryInclusions )) $smwgSubcategoryInclusions = 10;
// Normalizes the redirects, basically implements the sameAs semantics
// as defined by the mapping of redirects to sameAs.
// Disjunctive queries must be enabled for this feature to work. May cost performance
if (!isset( $smwgRedirectNormalization )) $smwgRedirectNormalization = TRUE;
// If true, sorting is enabled. May cost performance
if (!isset( $smwgSortingEnabled )) $smwgSortingEnabled = TRUE;
// Switches on or off all queries
if (!isset( $smwgInlineQueriesEnabled )) $smwgInlineQueriesEnabled = TRUE;

// first, we register a hook that will register a hook in the parser
global $wgHooks;
$wgHooks['ParserBeforeStrip'][] = 'smwfRegisterInlineQueries';

// This hook registers a hook in the parser
function smwfRegisterInlineQueries( $semantic, $wikipedia, $rules ) {
	global $wgParser;
	$wgParser->setHook( 'ask', 'smwfProcessInlineQueries' );
}

/**
 * Everything that is between <ask> and </ask> gets processed here
 * $text is the query in the query format described elsewhere
 * $param has the following parameters (all optional):
 * limit -- max number of answers. It will be capped by 500 anyway.
 * sort -- by which row to sort
 * order -- "ascending" (equal to "asc", default) or "descending" (equal to "desc" or "reverse")
 * sep -- in a list, what to use as a seperator
 * format -- either list (seperated by sep), ul (for unordered bullet list),
 *           ol (ordered and numbered list) or table
 * link -- either none, head or all. Makes links out of the results.
 *         Default is set by $smwgDefaultLinking
 * default -- if, for any reason, no result is being created, this will be returned
 */
function smwfProcessInlineQueries( $text, $param ) {
	$def = "";
	if (array_key_exists("default", $param)) {
		$def = $param["default"]; // TODO sanitize return value, bad code could be injected
	}
	$list = ""; // return value

	global $wgContLang, $wgParser;
	global $smwgMaximalLimit, $smwgStandardLimit, $smwgInnerQueriesEnabled, $smwgDisjunctiveQueriesEnabled, $smwgSortingEnabled, $smwgSubcategoryInclusions, $smwgInlineQueriesEnabled, $smwgDefaultLinking, $smwgRedirectNormalization;

	if (! $smwgInlineQueriesEnabled ) return $def;

	// this should also replace template parameters -- but it does not, and
	// I don't have the slightes clue why. The code afterwards was for testing. Don't
	// delete it until the templae / extension - interplay problem is fixed.
	$text = $wgParser->replaceVariables($text);
//	$tempOutputType = $wgParser->mOutputType;
//	$wgParser->mOutputType = OT_HTML;
//	$text = $wgParser->replaceVariables($text, end( $wgParser->mArgStack ), false);
//	$wgParser->mOutputType = $tempOutputType;
//	$text = $wgParser->replaceVariables($text, end( $wgParser->mArgStack ));
//	$list .= $text;
//	return $list;

	if ( $smwgInnerQueriesEnabled ) {
		$new_text = "";
		$inner_texts = explode( "<q>" , $text );
		foreach ($inner_texts as $inner) {
			if ( ereg( "</q>", $inner ) ) {
				list( $query , $rest ) = explode( "</q>", $inner, 2 );
				$inner_param = array( "sep" => "||" , "link" => "none" );
				$new_text .= smwfProcessInlineQueries( $query , $inner_param );
				$new_text .= $rest;
			} else {
				$new_text .= $inner;
			}
		}
		$text = $new_text;
	}

	$temp = ereg_replace("^[\n ]*\[\[", "", $text); // strip beginning
	$temp = ereg_replace("\]\][\n ]*$", "", $temp); // strip end
	$conjunctives = spliti("\]\][\n ]*\[\[", $temp); // explode

	$has_condition = FALSE;
	$columns = array();
	$column_is_attribute = array();
	$column_typehandler = array();
	$headers = array();
	$rename = array();
	$rename_count = 0;
	$category_count = 0;
	$sql_conditions = "";
	$sql_select = array( 'page_title' , 'page_namespace' );
	$fixed_subject = FALSE;

	$dbr =& wfGetDB( DB_SLAVE );
	if (!$dbr) return $def; // Maybe there should be an error message?
	// check if the MySQL version is 4.0.x. For < 4.0 we don't even care about,
	// because Mediawiki deprecated < 4.0 anyway. But some of the queries here
	// are written for >= 4.1.x, so we need to write them differently for 4.0,
	// and thus we need this information.
	$mysql40 = (substr($dbr->getServerVersion(), 0, 3) == "4.0");

	$sql_tables = $dbr->tableName( 'page' );

	foreach ($conjunctives as $conjunctive) {
		$attribute_condition = FALSE;
		$attribute_comparator = "=";
		$attribute_comparator_text = $attribute_comparator;
		$catregex = "^" . $wgContLang->getNsText(NS_CATEGORY) . ":";
		if (ereg($catregex . "[^:]", $conjunctive)) { // Category
			$categorylist = ereg_replace($catregex, "", $conjunctive);
			$cattemp = 0;
			if ( $smwgDisjunctiveQueriesEnabled ) {
				$cattemp = explode("||", $categorylist);
			} else {
				$cattemp = array( $categorylist );
			}
			$categories = array();
			foreach ($cattemp as $cat) {
				$categories[] = smwfNormalizeTitle($cat);
			}
			if ( $smwgSubcategoryInclusions > 0) {
				$categories = smwfIncludeSubcategories( $categories, $smwgSubcategoryInclusions );
			}
			if ($has_condition) $sql_conditions .= " AND ";
			$has_condition = TRUE;
			$firstobject = TRUE;
			$sql_tables .= ", ". $dbr->tableName('categorylinks') ." AS c". $category_count;
			$sql_conditions.=" page_id = c". $category_count . ".cl_from AND";
			foreach ($categories as $category) {
				if ($firstobject) {
					$sql_conditions .= " (";
					$firstobject = FALSE;
				} else {
					$sql_conditions .= " OR ";
				}
				$sql_conditions .= "c". $category_count .".cl_to=". $dbr->addQuotes($category);
			}
			$sql_conditions .= ") ";
			$category_count++;

		} elseif (ereg("::\*", $conjunctive)) { // querying relation
			$render = "";
			if (ereg("\|", $conjunctive)) {
				list($rest, $render) = explode("|", $conjunctive, 2); // TODO sanitize render. Could inject bad stuff
				$relation = ereg_replace("::\*$", "", $rest);
			} else {
				$relation = ereg_replace("::\*$", "", $conjunctive);
				$render = $relation;
			}
			$relation = smwfNormalizeTitle( $relation );
			$headers[$relation] = $render;
			$columns[] = $relation;
			$column_is_attribute[$relation] = FALSE;
			$rename[$relation] = "R".$rename_count++;
			$rename[$wgContLang->getNsText(SMW_NS_RELATION) .":". $relation] = $rename[$relation];
			$sql_tables .= ", ". $dbr->tableName('smw_relations') ." AS ". $rename[$relation];
			$sql_select[] = $rename[$relation].".object_title AS ". $rename[$relation]."o";
			if ($has_condition) $sql_conditions .= " AND ";
			$has_condition = TRUE;
			$sql_conditions .= $rename[$relation] .".relation_title=" . $dbr->addQuotes($relation) . " AND ". $rename[$relation] .".subject_id = page_id ";

		} elseif (ereg(":=\*", $conjunctive)) { // querying attribute
			$render = "";
			if (ereg("\|", $conjunctive)) {
				list($rest, $render) = explode("|", $conjunctive, 2); // TODO sanitize render. Could inject bad stuff
				$relation = ereg_replace(":=\*$", "", $rest);
			} else {
				$relation = ereg_replace(":=\*$", "", $conjunctive);
				$render = $relation;
			}
			$relation = smwfNormalizeTitle( $relation );
			$headers[$relation] = $render;
			$columns[] = $relation;
			$column_is_attribute[$relation] = TRUE;
			$column_typehandler[$relation] = &SMWGetTypeHandler($relation);
			$rename[$relation] = "R".$rename_count++;
			$rename[$wgContLang->getNsText(SMW_NS_ATTRIBUTE) .":". $relation] = $rename[$relation];
			$sql_tables .= ", ". $dbr->tableName('smw_attributes') ." AS ". $rename[$relation];
			if ($column_typehandler[$relation]->isNumeric()) {
				$sql_select[] = $rename[$relation].".value_num AS ". $rename[$relation]."o";
			} else {
				$sql_select[] = $rename[$relation].".value_xsd AS ". $rename[$relation]."o";
			}
			if ($has_condition) $sql_conditions .= " AND ";
			$has_condition = TRUE;
			$sql_conditions .= $rename[$relation] .".attribute_title=" . $dbr->addQuotes($relation) . " AND ". $rename[$relation] .".subject_id = page_id ";

		} elseif (ereg("::", $conjunctive)) { // conditional relation
			list($relation, $objectlist) = explode("::", $conjunctive, 2);
			$relation = smwfNormalizeTitle( $relation );
			if ( $smwgDisjunctiveQueriesEnabled ) {
				$objects_temp = explode("||", $objectlist);
			} else {
				$objects_temp = array( $objectlist );
			}
			$objects = array();
			foreach ( $objects_temp as $object ) {
				$objects[] = smwfNormalizeTitle( $object );
			}
			if ( $smwgRedirectNormalization ) {
				$objects = smwfNormalizeRedirects( $objects );
			}
			if ($has_condition) $sql_conditions .= " AND ";
			$has_condition = TRUE;
			$sql_tables .= ", ". $dbr->tableName('smw_relations') ." AS c". $category_count;
			$sql_conditions.=" page_id = c". $category_count . ".subject_id AND";
			$firstobject = TRUE;
			foreach ($objects as $object) {
				if ($firstobject) {
					$sql_conditions .= " (";
					$firstobject = FALSE;
				} else {
					$sql_conditions .= " OR ";
				}
				$sql_conditions.= "(c". $category_count .".relation_title=". $dbr->addQuotes($relation) ." AND c". $category_count .".object_title=". $dbr->addQuotes( $object ) .")";
			}
			$sql_conditions .= ") ";
			$category_count++;

		} elseif (ereg(":=>", $conjunctive)) { // conditional attribute min
			$attribute_condition = TRUE;
			$attribute_comparator = ">";
			$attribute_comparator_text = "=>";

		} elseif (ereg(":=<", $conjunctive)) { // conditional attribute max
			$attribute_condition = TRUE;
			$attribute_comparator = "<";
			$attribute_comparator_text = "=<";

		} elseif (ereg(":=", $conjunctive)) { // conditional attribute exact
			$attribute_condition = TRUE;
			$attribute_comparator = "=";
			$attribute_comparator_text = $attribute_comparator;

		} else { // there is a fixed subject
			if ( $smwgDisjunctiveQueriesEnabled ) {
				$subjects_temp = explode("||", $conjunctive);
			} else {
				$subjects_temp = array( $conjunctive );
			}
			$subjects = array();
			foreach ($subjects_temp as $subject) {
				$subjects[] = smwfNormalizeTitle( $subject );
			}
			if ( $smwgRedirectNormalization ) {
				$subjects = smwfNormalizeRedirects( $subjects );
			}
			if ($has_condition) $sql_conditions .= " AND ";
			$has_condition = TRUE;
			$sql_conditions .= " (";
			$firstobject = TRUE;
			foreach ($subjects as $subject) {
				if ($firstobject) {
					$firstobject = FALSE;
				} else {
					$sql_conditions .= " OR ";
				}
				$sql_conditions.=" page_title=". $dbr->addQuotes($subject) ." ";
			}
			$sql_conditions .= ") ";
			$fixed_subject = TRUE;
		}

		if ($attribute_condition) {
			list($relation, $object) = explode(":" . $attribute_comparator_text, $conjunctive, 2);
			$relation = smwfNormalizeTitle( $relation );
			$column_typehandler[$relation] = &SMWGetTypeHandler($relation);
			$th = $column_typehandler[$relation];
			if ($has_condition) $sql_conditions .= " AND ";
			$has_condition = TRUE;
			$sql_tables .= ", ". $dbr->tableName('smw_attributes') ." AS c". $category_count;
			$sql_conditions.=" page_id = c". $category_count . ".subject_id AND c". $category_count . ".attribute_title=". $dbr->addQuotes($relation);
			if ($th->isNumeric()) {
				$pv = $th->processValue($object);
				if (array_key_exists("NUM", $pv)) {
					$sql_conditions.= " AND c". $category_count . ".value_num". $attribute_comparator . $pv['NUM'] ." ";
				} else {
					$sql_conditions.= " AND c". $category_count . ".value_xsd". $attribute_comparator . $dbr->addQuotes($object) ." ";
				}
			} else {
				$sql_conditions.= " AND c". $category_count . ".value_xsd". $attribute_comparator . $dbr->addQuotes($object) ." ";
			}
			$category_count++;
		}
	}

	$categorylinks = $dbr->tableName( 'categorylinks' );

	$limit = $smwgStandardLimit;
	if (array_key_exists("limit", $param)) {
		$set_limit = $param["limit"] + 0; // conversion to int
		if (is_int($set_limit)) {
			$limit = $set_limit;
		}
		if ($limit > $smwgMaximalLimit) {
			$limit=$smwgMaximalLimit;
		}
	}

	$order = "page_title";
	if (array_key_exists("sort", $param)) {
		if (array_key_exists(ucfirst($param["sort"]), $rename)) {
			$order = $rename[ ucfirst($param["sort"]) ] ."o";
		}
	}
	if (array_key_exists("order", $param)) {
		if (("descending"==$param["order"])||("reverse"==$param["order"])||("desc"==$param["order"])) {
			$order .= " DESC ";
		}
	}

	if ( $smwgSortingEnabled ) {
		$sql_options = array( 'LIMIT' => $limit, 'ORDER BY' => $order );
	} else {
		$sql_options = array( 'LIMIT' => $limit );
	}

	// switch on the following line to get the query -- useful for debugging and understanding this code
	//$def .= $list .= "SELECT " . implode(",",$sql_select) . " FROM $sql_tables WHERE $sql_conditions <br />";


	$res = $dbr->select( // make the query
		$sql_tables,
		"DISTINCT ". implode(",", $sql_select),
		array( $sql_conditions ),
		"SMW_InlineQuery" ,
		$sql_options );

	$format = "list";
	if (array_key_exists("format", $param)) {
		$format = $param["format"];
		if (($format != "ul") && ($format != "ol") && ($format != "list") && ($format != "table"))
			$format = "list"; // If it is an unknown format, default to list again
	}
	$sep = ", ";
	if (array_key_exists("sep", $param)) {
		$sep = $param["sep"]; // TODO malicious code could be injected here. Use some kind of sanitizer
	}
	$fieldsep = $sep;
	$fieldsepnum = $sep;

	if ((count($columns)>1) || ((count($columns)==1) && (!$fixed_subject))) {
		// the answer format must be a table if the answer cannot be
		// represented in one dimension
		$format = "table";
	}

	$tableid = 0;
	if ($format=="table") {
		$format = "table";
		$sep = "</td>"."\n"."  </tr>"."\n"."  <tr>"."\n"."    <td>";
		$fieldsep = "</td>"."\n".'    <td>';
		$fieldsepnum = "</td>"."\n".'    <td align="right">';
		$list .= '<table class="smwtable" id="table'. $tableid++ .'">'."\n"."  <tr>"."\n"."    <th> &nbsp; </th>"."\n";
		foreach ($columns as $relation) {
			$list .= "    <th>".$headers[$relation]."</th>"."\n";
		}
		$list .= "  </tr>"."\n"."  <tr>"."\n"."    <td>";
	}

	if (($format == "ul") || ($format == "ol")) {
		$list .= "<" . $format . ">" . "\n" . "  <li>";
		$sep = "</li>"."\n"."  <li>";
	}

	$link_head = ($smwgDefaultLinking != "none");
	$link_body = (($format == "list") && ($smwgDefaultLinking != "none")) || ($smwgDefaultLinking == "all");
	if (array_key_exists("link", $param)) {
		if ($param["link"] == "head") {
			$link_head = TRUE;
			$link_body = FALSE;
		} elseif ($param["link"] == "all") {
			$link_head = TRUE;
			$link_body = TRUE;
		} elseif ($param["link"] == "none") {
			$link_head = FALSE;
			$link_body = FALSE;
		}
	}

	if ( $res ) {
		$num = $dbr->numRows( $res );
		if (0==$num) return $def;
		$first = TRUE;
		while ( $res && $row = $dbr->fetchRow( $res )) {
			if ( array_key_exists('page_title', $row) ) {
				if ((!$fixed_subject) || ($format == "table")) {
					if ( $first ) {
						$first = FALSE;
					} else {
						$list .= $sep;
					}
					$title = Title::newFromText($row["page_title"] , $row["page_namespace"]);
					if ($title == NULL) return $def;
					if ($link_head) {
						$list .= '<a href="'. $title->getLocalURL() .'" title="'. $title->getText() .'">'. $title->getText() .'</a>';
					} else {
						$list .= $title->getText();
					}
				}
				foreach ($columns as $relation) {
					if ( $first ) {
						$first = FALSE;
					} else {
						if ($column_is_attribute[$relation]) {
							if ($column_typehandler[$relation]->isNumeric()) {
								$list .= $fieldsepnum;
							} else {
								$list .= $fieldsep;
							}
						} else {
							$list .= $fieldsep;
						}
					}
					$value = "";
					if ( array_key_exists ($rename[$relation].'o', $row) ) {
						$value = $row[$rename[$relation].'o'];
					}
					if (($link_body) && (!$column_is_attribute[$relation])) {
						$title = Title::newFromText( $value );
						if ($title == NULL) return $def;
						if ($title->exists()) {
							$classnew = '';
						} else {
							$classnew = 'class="new" ';
						}
						$list .= '<a href="'. $title->getLocalURL() .'" '. $classnew .'title="'. $title->getText() .'">'. $title->getText() .'</a>';
					} else {
						if (!$column_is_attribute[$relation]) {
							$title = Title::newFromText( $value );
							if ($title == NULL) return $def;
							$list .= $title->getText();
						} else {
							$th = $column_typehandler[$relation];
							$pv = $th->processValue($value);
							if ($th->isNumeric()) {
								$unit = $th->getUnits();
								$list .= SMWNumberFormat($pv['XSD']);
								if ($unit['STDUNIT']!="")
									$list .= " " . $unit['STDUNIT'];
							} else {
								$list .= $pv['USER'];
							}
						}
					}
				}
			}
		}

		if ( $res ) {
			$dbr->freeResult( $res );
		}
	} else {
		return $def; // There was an error, and we are quiet. Maybe some kind of erros message should be displayed?
	}

	if (($format == "ul") || ($format == "ol")) {
		$list .= "</li>"."\n"."</".$format.">"."\n";
	}

	if ($format=="table") {
		$list .= "</td>"."\n"."  </tr>"."\n"."</table>";
	}

	return $list;
}

/**
 * Turns an array of articles into an array of all these articles and
 * and the transitive closure of all redirects from and to this articles.
 * Or, simply said: it gets all aliases of what you put in.
 */
function smwfNormalizeRedirects( $articles ) {
	$dbr =& wfGetDB( DB_SLAVE );
	$stable = FALSE;
	$check_articles = array_diff( $articles , array() ); // Copies the array
	while (!$stable) {
		$new_articles = array();
		foreach ( $check_articles as $article ) {
			// there...
			$res = $dbr->select(
				array( 'page' , 'pagelinks' ),
				array( 'pl_title' ),
				array(  'page_title = '. $dbr->addQuotes( $article ),
						'page_is_redirect = 1',
						'page_id = pl_from' ) ,
						"SMW_NormalizeRedirects" );
			if ( $res ) {
				while ( $res && $row = $dbr->fetchRow( $res )) {
					if ( array_key_exists( 'pl_title', $row) ) {
						$new_article = $row[ 'pl_title' ];
						if (!in_array( $new_article , $articles)) {
							$new_articles[] = $new_article;
						}
					}
				}
			}
			$dbr->freeResult( $res );

			// ... and back again
			$res = $dbr->select(
				array( 'page' , 'pagelinks' ),
				array( 'page_title' ),
				array(  'pl_title = '. $dbr->addQuotes( $article ),
						'page_is_redirect = 1',
						'page_id = pl_from' ) ,
						"SMW_NormalizeRedirects" );
			if ( $res ) {
				while ( $res && $row = $dbr->fetchRow( $res )) {
					if ( array_key_exists( 'page_title', $row) ) {
						$new_article = $row[ 'page_title' ];
						if (!in_array( $new_article , $articles)) {
							$new_articles[] = $new_article;
						}
					}
				}
			}
			$dbr->freeResult( $res );
		}
		$stable = (count($new_articles)==0);
		$articles = array_merge( $articles, $new_articles );
		$check_articles = array_diff( $new_articles , array() );
	}
	return $articles;
}

/**
 * Turns an array of categories to an array of categories and its subcategories.
 * The number of relations followed is given by $levels.
 */
function smwfIncludeSubcategories( $categories, $levels ) {
	$dbr =& wfGetDB( DB_SLAVE );
	$checkcategories = array_diff($categories, array()); // Copies the array
	for ($level=$levels; $level>0; $level--) {
		$newcategories = array();
		foreach ($checkcategories as $category) {
			$res = $dbr->select( // make the query
				array( 'categorylinks', 'page' ),
				array( 'page_title' ),
				array(  'cl_from = page_id ',
						'page_namespace = '. NS_CATEGORY,
						'cl_to = '. $dbr->addQuotes($category) ) ,
						"SMW_SubCategory" );
			if ( $res ) {
				while ( $res && $row = $dbr->fetchRow( $res )) {
					if ( array_key_exists( 'page_title' , $row) ) {
						$new_category = $row[ 'page_title' ];
						if (!in_array($new_category, $categories)) {
							$newcategories[] = smwfNormalizeTitle($new_category);
						}
					}
				}
				$dbr->freeResult( $res );
			}
		}
		if (count($newcategories) == 0) {
			return $categories;
		} else {
			$categories = array_merge($categories, $newcategories);
		}
		$checkcategories = array_diff($newcategories, array());
	}
	return $categories;
}

/**
 * Takes a text and turns it safely into its DBKey.
 * If it fails to do so, the text is returned unchanged.
 */
function smwfNormalizeTitle( $text ) {
	$t = Title::newFromText( $text );
	if ($t != NULL) {
		return $t->getDBkey();
	}
	return $text;
}
?>
