<?php
/**
 * This file encapsulates methods used for accessing
 * and storing semantic data.
 *
 * @author Markus Krtzsch
 * @author Klaus Lassleben
 */
 
	global $wgDBprefix;
	$smwgOldTableName = $wgDBprefix . 'semantic_relations';
	
	$smwgRelTableName = $wgDBprefix . 'smw_relations';
	$smwgAttTableName = $wgDBprefix . 'smw_attributes';
	$smwgSpcTableName = $wgDBprefix . 'smw_specialprops';

	/**
	 * This method writes a single relation into the database.
	 * $subject_title and $object_title are expected as title
	 * objects for the respective articles. $rel_name is the
	 * string label of the relation, without namespace label.
	 */
	function SMWStoreRelation($subject_title, $rel_title, $object_title) {
		global $smwgRelTableName;
		
		$fname = 'SMW::StoreRelation';
		$db =& wfGetDB( DB_MASTER );
		$sql = 'INSERT INTO ' . $smwgRelTableName . '
			(subject_id, subject_namespace, subject_title, relation_title, object_namespace, object_title)
			VALUES ('
			. $db->addQuotes($subject_title->getArticleID()) . ',' 
			. $db->addQuotes($subject_title->getNamespace()) . ',' 
			. $db->addQuotes($subject_title->getDBkey()) . ',' 
			. $db->addQuotes($rel_title->getDBkey()) . ',' 
			. $db->addQuotes($object_title->getNamespace()) . ',' 
			. $db->addQuotes($object_title->getDBkey()) . ')';
		$res = $db->query( $sql, $fname );
		return ( false !== $res );
	}
	
	/**
	 * This method writes a single relation into the database.
	 * $subject_title is expected as title object of the article. 
	 * $att_name and $att_unit are the string labels of the 
	 * attribute-name, without namespace label, and of its unit.
	 * $att_type is the internal (international) typeid of the 
	 * type, and $value is a value-string.
	 */
	function SMWStoreAttribute($subject_title, $att_title, $att_unit, $att_type, $value, $value_num=NULL) {
		global $smwgAttTableName;
		
		if ($att_type=='') { // fallback to string if no datatype sepcified
			$att_type='string';
		}
		
		$fname = 'SMW::StoreAttribute';
		$db =& wfGetDB( DB_MASTER );
		$sql = 'INSERT INTO ' . $smwgAttTableName . '
			(subject_id, subject_namespace, subject_title, attribute_title, value_unit, value_datatype, value_xsd, value_num)
			VALUES ('
			. $db->addQuotes($subject_title->getArticleID()) . ',' 
			. $db->addQuotes($subject_title->getNamespace()) . ',' 
			. $db->addQuotes($subject_title->getDBkey()) . ',' 
			. $db->addQuotes($att_title->getDBkey()) . ',' 
			. $db->addQuotes($att_unit) . ',' 
			. $db->addQuotes($att_type) . ',' 
			. $db->addQuotes($value) . ',' 
			. $db->addQuotes($value_num) . ')';
		$res = $db->query( $sql, $fname );
		return ( false !== $res );
	}
	
	/**
	 * This method writes a single special property into the 
	 * database. $subject_title is expected to be a title
	 * object. $prop_name the index of some special property.
	 * $value is a string. This method also preprocesses some
	 * values, e.g. to remove implied namespace prefixes.
	 */
	function SMWStoreSpecialProperty($subject_title, $prop_name, $value) {
		global $smwgSpcTableName;
		
		if ($prop_name == SMW_SP_HAS_TYPE) { // remove any namespace prefix from $value
			$vtitle = Title::newFromText($value);
			$value = $vtitle->getText();
		}
		
		$fname = 'SMW::StoreSpecialProperty';
		$db =& wfGetDB( DB_MASTER );
		$sql = 'INSERT INTO ' . $smwgSpcTableName . '
			(subject_id, subject_namespace, subject_title, property_id, value_string)
			VALUES ('
			. $db->addQuotes($subject_title->getArticleID()) . ',' 
			. $db->addQuotes($subject_title->getNamespace()) . ',' 
			. $db->addQuotes($subject_title->getDBkey()) . ',' 
			. $db->addQuotes($prop_name) . ',' 
			. $db->addQuotes($value) . ')';
		$res = $db->query( $sql, $fname );
		return ( false !== $res );
	}
	
	/**
	 *  This method cleans up relations for a specified subject.
	 *  The subject is to be given as a title-object.
	 */
	function SMWDeleteRelations($subject_title)
	{
		global $smwgRelTableName;
		
		$fname = 'SMW::DeleteRelations';
		$db =& wfGetDB( DB_MASTER );
		
		$sql = 'DELETE FROM ' . $smwgRelTableName 
		       . ' WHERE subject_id=' . $db->addQuotes($subject_title->getArticleID());
		$res = $db->query( $sql, $fname );
		if($db->numRows( $res ) > 0)
		{
			$db->freeResult($res);
		}
		return true;
	}
	
	/**
	 *  This method cleans up attributes for a specified subject.
	 *  The subject is to be given as a title-object.
	 */
	function SMWDeleteAttributes($subject_title)
	{
		global $smwgAttTableName;
		
		$fname = 'SMW::DeleteAttributes';
		$db =& wfGetDB( DB_MASTER );
		
		$sql = 'DELETE FROM ' . $smwgAttTableName 
		       . ' WHERE subject_id=' . $db->addQuotes($subject_title->getArticleID());
		$res = $db->query( $sql, $fname );
		if($db->numRows( $res ) > 0)
		{
			$db->freeResult($res);
		}
		return true;
	}
	
	/**
	 *  This method cleans up special properties for a specified subject.
	 *  The subject is to be given as a title-object.
	 */
	function SMWDeleteSpecialProperties($subject_title)
	{
		global $smwgSpcTableName;
		
		$fname = 'SMW::DeleteSpecialProperties';
		$db =& wfGetDB( DB_MASTER );
		
		$sql = 'DELETE FROM ' . $smwgSpcTableName 
		       . ' WHERE subject_id=' . $db->addQuotes($subject_title->getArticleID());
		$res = $db->query( $sql, $fname );
		if($db->numRows( $res ) > 0)
		{
			$db->freeResult($res);
		}
		return true;
	}
	
	/**
	*  This method fetches relations for a specified subject, relation, and object.
	*  If some (but not all) of these are NULL, the search fetches all triples
	*  matching the remaining values. The result corresponds to the layout of the
	*  database table for relations.
	*/
	function SMWGetRelations($subject_title,$relation_title,$object_title,$fuzzysearch=false)
	{
		global $smwgRelTableName;

		$fname = 'SMW::GetRelations';
		$db =& wfGetDB( DB_MASTER );
		
		$sql='';
		if ($subject_title !== NULL) {
			$sql.='subject_id=' . $db->addQuotes($subject_title->getArticleID());
		}
		if ($relation_title !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			if ($fuzzysearch)
				$sql.='relation_title LIKE' . $db->addQuotes('%' . $relation_title->getDBkey() . '%');
			else
				$sql.='relation_title =' . $db->addQuotes($relation_title->getDBkey());
		}
		if ($object_title !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			$sql.='object_namespace=' . $db->addQuotes($object_title->getNamespace());
			if ($fuzzysearch)
				$sql.= ' AND object_title LIKE' . $db->addQuotes('%' . $object_title->getDBkey() . '%');
			else
				$sql.= ' AND object_title = ' . $db->addQuotes($object_title->getDBkey());
		}
		
		if ($sql=='') {
			return false; //do not execute queries for Everything
		}
		
		$sql = 'SELECT subject_id, relation_title, object_namespace, object_title FROM ' .
		       $smwgRelTableName . ' WHERE ' . $sql;
		$res = $db->query( $sql, $fname );
		
		$result = array();
		if($db->numRows( $res ) > 0)
		{
			$row = $db->fetchObject($res);
			while($row)
			{
				$result[]=array($row->subject_id,$row->relation_title,$row->object_namespace,$row->object_title);
				$row = $db->fetchObject($res);
			}
		}
		$db->freeResult($res);
		
		return $result;
	}
	
	/**
	*  This method fetches relations for a specified subject, attribute, unit, type, and
	*  value. If some (but not all) of these are NULL, the search fetches all triples
	*  matching the remaining values. The result corresponds to the layout of the
	*  database table for attributes.
	*/
	function SMWGetAttributes($subject_title, $att_title, $att_unit, $att_type, $value, $fuzzysearch=false)
	{
		global $smwgAttTableName;

		$fname = 'SMW::GetAttributes';
		$db =& wfGetDB( DB_MASTER );
		
		$sql='';
		if ($subject_title !== NULL) {
			$sql.='subject_id=' . $db->addQuotes($subject_title->getArticleID());
		}
		if ($att_title !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			if ($fuzzysearch)
				$sql.='attribute_title LIKE' . $db->addQuotes('%' . $att_title->getDBkey() .'%');
			else
				$sql.='attribute_title=' . $db->addQuotes($att_title->getDBkey());
		}
		if ($att_unit !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			$sql.='value_unit=' . $db->addQuotes($att_unit);
		}
		if ($att_type !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			$sql.='value_datatype=' . $db->addQuotes($att_type);
		}
		if ($value !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			if ($fuzzysearch)
				$sql.='value_xsd LIKE' . $db->addQuotes($value. '%');
			else
				$sql.='value_xsd = ' . $db->addQuotes($value);
		}
		
		if ($sql=='') {
			return false; //do not execute queries for Everything
		}
		
		$sql = 'SELECT subject_id, attribute_title, value_unit, value_datatype, value_xsd FROM ' . 
		       $smwgAttTableName . ' WHERE ' . $sql;
		$res = $db->query( $sql, $fname );
		
		$result = array();
		if($db->numRows( $res ) > 0)
		{
			$row = $db->fetchObject($res);
			while($row)
			{
				$result[]=array($row->subject_id,$row->attribute_title,$row->value_unit,$row->value_datatype,$row->value_xsd);
				$row = $db->fetchObject($res);
			}
		}
		$db->freeResult($res);
		
		return $result;
	}
	
	
	/**
	*  This method fetches special properties for a specified subject, property,
	*  and value. If some (but not all) of these are NULL, the search fetches 
	*  all triples matching the remaining values. The result corresponds to the 
	*  layout of the database table for special properties. Note that property
	*  is an integer index.
	*/
	function SMWGetSpecialProperties($subject_title,$property,$value)
	{
		global $smwgSpcTableName;

		$fname = 'SMW::GetSpecialProperties';
		$db =& wfGetDB( DB_MASTER );
		
		$sql='';
		if ($subject_title !== NULL) {
			$sql.='subject_id=' . $db->addQuotes($subject_title->getArticleID());
		}
		if ($property !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			$sql.='property_id=' . $db->addQuotes($property);
		}
		if ($value !== NULL) {
			if ($sql!='') {$sql.=' AND ';}
			$sql.='value_string=' . $db->addQuotes($value);
		}
		
		if ($sql=='') {
			return false; //do not execute queries for Everything
		}
		
		$sql = 'SELECT subject_id, property_id, value_string FROM ' .
		       $smwgSpcTableName . ' WHERE ' . $sql;
		$res = $db->query( $sql, $fname );
		
		$result = array();
		if($db->numRows( $res ) > 0)
		{
			$row = $db->fetchObject($res);
			while($row)
			{
				$result[]=array($row->subject_id,$row->property_id,$row->value_string);
				$row = $db->fetchObject($res);
			}
		}
		$db->freeResult($res);
		
		return $result;
	}
	
	
	/**
	 * This method changes the subject of all data with a specified
	 * subject. It is required for moving annotations together with their
	 * articles. As usual, input values are expected to be titles.
	 * 
	 * This method requires the ArticleIDs of all titles to be available.
	 * Note that this is typically not the case during normal article move 
	 * operations, but that the IDs do not change in this case anyway.
	 * 
	 * Due to its lack of use cases, this method might be removed in the future.
	 */
	function SMWMoveAnnotations($old_stitle,$new_stitle)
	{
		$old_id = $old_stitle->getArticleID();
		$new_id = $new_stitle->getArticleID();
		
		if (($old_id == $new_id) || ($old_id == 0) || ($new_id == 0)) { 
			return true;
		}
		
		global $smwgRelTableName, $smwgAttTableName, $smwgSpcTableName;
		
		$fname = 'SMW::MoveAnnotations';
		$db =& wfGetDB( DB_MASTER );

		$sql = "UPDATE $smwgRelTableName SET subject_id=" . $db->addQuotes( $new_id ) . 
			' WHERE subject_id=' . $db->addQuotes( $old_id );
		$db->query( $sql, $fname );
		$sql = "UPDATE $smwgAttTableName SET subject_id=" . $db->addQuotes( $new_id ) . 
			' WHERE subject_id=' . $db->addQuotes( $old_id );
		$db->query( $sql, $fname );
		$sql = "UPDATE $smwgSpcTableName SET subject_id=" . $db->addQuotes( $new_id ) . 
			' WHERE subject_id=' . $db->addQuotes( $old_id );
		$db->query( $sql, $fname );
		
		return true;
	}
	
	
	/**
	 * This method creates database tables to store the triples internally.
	 */
	function SMWMakeSemanticTables() {
		global $wgDBname, $smwgRelTableName, $smwgAttTableName, $smwgSpcTableName;

		$fname = 'SMW::MakeSemanticTables';
		$db =& wfGetDB( DB_MASTER );

		// create relation table
		$sql = 'CREATE TABLE ' . $wgDBname . '.' . $smwgRelTableName . '
				( subject_id         INT(8) UNSIGNED NOT NULL,
				  subject_namespace  INT(11) NOT NULL,
				  subject_title      VARCHAR(255) NOT NULL,
				  relation_title     VARCHAR(255) NOT NULL,
				  object_namespace   INT(11) NOT NULL,
				  object_title       VARCHAR(255) NOT NULL
				) TYPE=innodb';
		$res = $db->query( $sql, $fname );

		// create attribute table
		$sql = 'CREATE TABLE ' . $wgDBname . '.' . $smwgAttTableName . '
				( subject_id INT(8) UNSIGNED NOT NULL,
				  subject_namespace  INT(11) NOT NULL,
				  subject_title      VARCHAR(255) NOT NULL,
				  attribute_title    VARCHAR(255) NOT NULL,
				  value_unit         VARCHAR(63),
				  value_datatype     VARCHAR(31) NOT NULL,
				  value_xsd          VARCHAR(255) NOT NULL,
				  value_num          DOUBLE
				) TYPE=innodb';
		$res = $db->query( $sql, $fname );

		// create table for special properties
		$sql = 'CREATE TABLE ' . $wgDBname . '.' . $smwgSpcTableName . '
				( subject_id         INT(8) UNSIGNED NOT NULL,
				  subject_namespace  INT(11) NOT NULL,
				  subject_title      VARCHAR(255) NOT NULL,
				  property_id        SMALLINT NOT NULL,
				  value_string       VARCHAR(255) NOT NULL
				) TYPE=innodb';
		$res = $db->query( $sql, $fname );
		
		//add indices for new columns
		$sql = "ALTER TABLE $smwgRelTableName ADD INDEX ( `subject_id` )";
		$db->query( $sql, $fname );
		$sql = "ALTER TABLE $smwgRelTableName ADD INDEX ( `relation_title` )";
		$db->query( $sql, $fname );
		$sql = "ALTER TABLE $smwgRelTableName ADD INDEX ( `object_title` )";
		$db->query( $sql, $fname );				
		$sql = "ALTER TABLE $smwgAttTableName ADD INDEX ( `subject_id` )";
		$db->query( $sql, $fname );
		$sql = "ALTER TABLE $smwgAttTableName ADD INDEX ( `attribute_title` )";
		$db->query( $sql, $fname );		
		$sql = "ALTER TABLE $smwgAttTableName ADD INDEX ( `value_num` )";
		$db->query( $sql, $fname );		
		$sql = "ALTER TABLE $smwgAttTableName ADD INDEX ( `value_xsd` )";
		$db->query( $sql, $fname );		
		$sql = "ALTER TABLE $smwgSpcTableName ADD INDEX ( `subject_id` )";
		$db->query( $sql, $fname );

		return true;
	}
	
	/**
	 * This method checks for the existence of a table with the given name.
	 * TODO: should be replaced by Database->tableExists (which takes the unprefixed tablename)
	 */
	function SMWCheckForTable($tablename) {
		global $wgDBname;
		
		$fname = 'SMW::CheckForTable';
		$db =& wfGetDB( DB_MASTER );
		
		$sql = 'SHOW TABLES LIKE \'' . $tablename . '\'';
		$res = $db->query( $sql, $fname );
		if($db->numRows( $res ) > 0) {
			$db->freeResult($res);
			return true;
		}
		else {
			$db->freeResult($res);
			return false;
		}
	}
	
	/**
	 * This method checks for the existence of a certain column in a some table.
	 */
	function SMWCheckForTableColumn($tablename, $columnname) {
		global $wgDBname;
		
		$fname = 'SMW::CheckForTableRow';
		$db =& wfGetDB( DB_MASTER );
		
		
		
		$sql = 'SHOW COLUMNS FROM \'' . $tablename . '\'';
		$res = $db->query( $sql, $fname );
		if($db->numRows( $res ) > 0) {
			$db->freeResult($res);
			return true;
		}
		else {
			$db->freeResult($res);
			return false;
		}
	}
 
?>