<?php
/**
 * @author Markus Krötzsch
 *
 * This special page for MediaWiki implements an RDF-export of
 * semantic data, gathered both from the annotations in articles,
 * and from metadata already present in the database.
 */

if (!defined('MEDIAWIKI')) die();

require_once($smwgIP . '/includes/SMW_Storage.php');
require_once( "$IP/includes/SpecialPage.php" );

$wgExtensionFunctions[] = "wfExportRDFExtension";

function wfExportRDFExtension()
{
	smwfInitMessages(); // initialize messages, always called before anything else on this page
	SpecialPage::addPage( new SpecialPage('ExportRDF','',true,'doSpecialExportRDF',false) );
}


function doSpecialExportRDF($page = '')
{
	global $wgOut, $wgRequest, $wgUser;
	
	// check whether we already know what to export
	if ($page=='') { //try to get GET parameter
		$page = $wgRequest->getVal( 'page' );
	}
	
	if ($page=='') { //try to get POST list
		$pageblob = $wgRequest->getText( 'pages' );
		if ($pageblob!='') {
			$pages = explode( "\n", $pageblob );
		}
	} else {
		$pages = array($page);
	}
	
	if( $pages !== NULL ) {  // export to RDF
		$wgOut->disable();
		//ob_end_clean();
		ob_start();
		
		$recursive = $wgRequest->getText( 'recursive' );
		if (($recursive=='') || ((!$wgUser->isAllowed('delete')) && ($recursive>=0))) {
			$recursive = 0;
		}
		
		// only use rdf+xml mimetype if explicitly requested
		if ( $wgRequest->getVal( 'xmlmime' )=='rdf' ) { 
			header( "Content-type: application/rdf+xml; charset=UTF-8" );
		} else {
			header( "Content-type: application/xml; charset=UTF-8" );
		}
		
		$exRDF = new ExportRDF();
		$exRDF->printPages($pages,$recursive);
		return;
	}

	// nothing exported yet; show user interface:
	$html = '<form name="tripleSearch" action="" method="POST">' . "\n" .
	        //'<p><strong>' . wfMsg('smw_devel_warning') . '</strong></p>' . 
	        wfMsg('smw_exportrdf_docu') . "\n" .
	        '<textarea name="pages" cols="40" rows="10"></textarea><br />' . "\n";
	if ( $wgUser->isAllowed('delete') ) {
		$html .= '<input type="checkbox" name="recursive" value="1" id="rec"><label for="rec">' . wfMsg('smw_exportrdf_recursive') . '</label></input><br/><br/>';
	}
	$html .= "<input type=\"submit\"/>\n</form>";
	$wgOut->addHTML($html);
}





class ExportRDF
{
	/**#@+
	 * @access private
	 */

	/**
	 * The basic namespace for acrticles in this wiki (computed on creation).
	 * Version suitable for URLs.
	 */
	var $wiki_xmlns_url;
	
	/**
	 * The basic namespace for acrticles in this wiki (computed on creation).
	 * Version suitable for XML identifiers.
	 */
	var $wiki_xmlns_xml;
	
	/**
	 * The XML generated for describing semantic data
	 */
	var $XML_desc;
	/**
	 * The XML generated for declaring relations and attributes
	 */
	var $XML_decl;
	/**
	 * An array that keeps track of the elements for which we still need to
	 * write auxilliary definitions.
	 */
	var $element_queue;
	
	/**
	 * URL of this special page
	 */
	var $special_url;

	function ExportRDF()
	{
		global $smwgServer; // server address for URIs (without http://)
		global $wgServer;   // actual server address (with http://)
		global $wgScript;   // "/subdirectory/of/wiki/index.php"
		//@TODO: generate this properly, using $wgArticlePath
		$this->wiki_xmlns_url = $wgServer . $wgScript . '/';
		$this->wiki_xmlns_xml = 'http://' . $this->makeXMLExportId($smwgServer . $wgScript) . '/';
		
		$title = Title::makeTitle( NS_SPECIAL, 'ExportRDF' );
		$this->special_url = '&wikiurl;' . $title->getPrefixedURL();

		$this->element_queue = Array();
	}

	/**
	 * This function prints all selected pages. The parameter $recursion determines
	 * how referenced ressources are treated:
	 * '0' : add brief declarations for each
	 * '1' : add full descriptions for each, thus beginning real recursion (and 
	 *       probably retrieving the whole wiki ...)
	 * else: ignore them
	 */
	function printPages($pages,$recursion=1) {
		$this->printHeader();
		while (count($pages)>0) {
			foreach ( $pages as $page) {
				$this->printTriples($page);
			}
			$pages = array();
			if (1==$recursion) {
				foreach ( $this->element_queue as $page => $open ) {
					if ($open) {
						$pages[] = $page;
						$this->element_queue[$page] = false; // prevent loops even if this code is buggy
					}
				}
			}
		}

		if (0==$recursion) {
			print "\t<!-- auxilliary definitions -->\n";
			foreach ( $this->element_queue as $page => $open ) {
				if ($open) $this->printTriples($page,false);
			}
		}
		
		$this->printFooter();
	}
	
	/* Functions for exporting RDF */

	function printHeader() {
		global $wgContLang;
		
		print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" .
			"<!DOCTYPE rdf:RDF[\n" .
			"\t<!ENTITY rdf 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'>\n" .
			"\t<!ENTITY rdfs 'http://www.w3.org/2000/01/rdf-schema#'>\n" .
			"\t<!ENTITY owl 'http://www.w3.org/2002/07/owl#'>\n" .
			"\t<!ENTITY smw 'http://smw.ontoware.org/2005/smw#'>\n" .
			"\t<!ENTITY smwdt 'http://smw.ontoware.org/2005/smw-datatype#'>\n" .
			"\t<!ENTITY wiki '" . $this->wiki_xmlns_xml .  "'>\n" .
			"\t<!ENTITY thing '" . $this->wiki_xmlns_xml . '_' .  "'>\n" .
			"\t<!ENTITY relation '" . $this->wiki_xmlns_xml . '_' .
			$this->makeXMLExportId(urlencode(str_replace(' ', '_', $wgContLang->getNsText(SMW_NS_RELATION) . ':'))) .  "'>\n" .
			"\t<!ENTITY attribute '" . $this->wiki_xmlns_xml . '_' . 
			$this->makeXMLExportId(urlencode(str_replace(' ', '_', $wgContLang->getNsText(SMW_NS_ATTRIBUTE) . ':'))) .  "'>\n" .
			"\t<!ENTITY wikiurl '" . $this->wiki_xmlns_url .  "'>\n" .
			"]>\n\n" .
			"<rdf:RDF\n" .
			"\txmlns:rdf=\"&rdf;\"\n" .
			"\txmlns:rdfs=\"&rdfs;\"\n" .
			"\txmlns:owl =\"&owl;\"\n" .
			"\txmlns:smw=\"&smw;\"\n" .
			"\txmlns:wiki=\"&wiki;\"\n" .
			"\txmlns:thing=\"&thing;\"\n" .
			"\txmlns:relation=\"&relation;\"\n" .
			"\txmlns:attribute=\"&attribute;\">\n" .
			"\t<!-- reference to the Semantic MediaWiki schema -->\n" .
			"\t<rdf:Description rdf:about=\"&smw;hasArticle\">\n" .
			"\t\t<rdfs:isDefinedBy rdf:resource=\"http://smw.ontoware.org/2005/smw\"/>\n" .
			"\t</rdf:Description>\n" .
			"\t<rdf:Description rdf:about=\"&smw;Thing\">\n" .
			"\t\t<rdfs:isDefinedBy rdf:resource=\"http://smw.ontoware.org/2005/smw\"/>\n" .
			"\t</rdf:Description>\n" .
			"\t<!-- exported page data -->\n";
	}

	function printFooter() {
		print '</rdf:RDF>';
	}
	
	function printTriples($page, $fullexport=true) {
		$title = Title::newFromText($page);	
		if ($title === NULL) { return; } //this should not happen ...
		
		//// == normalize title text ==
		// This deserves some explanation: we process the given page name via
		// a title object since this eliminates acceptable discrepancies in 
		// the user input. However, we have to recover the fragment part as
		// well since we use fragments to encode units for attributes, which
		// is required especially when calling this method recursively. For the
		// recursive case, the normlization step is not required, since it was 
		// already performed in getURIs. However, we need the title for our
		// database access anyway, and thus we do not loose much. It might be
		// questionable whether the normalization of the fragement part is 
		// actually desirable, since it is normally different from the unit string
		// that the datatype object returns for storing in the internal database.
		// Maybe we should do a title-normalization of the internal unit strings
		// when storing them.
		
		$page = $title->getText();
		$fragment = $title->getFragment();
		if( '' != $fragment ) {
			$fragment = '#' . $fragment;
		}
		$pageURIs = $this->getURIs($page . $fragment, $title->getNamespace());
		$pageID = $title->getArticleID();
		
		switch ($title->getNamespace()) {
			case SMW_NS_RELATION:
				$type = 'owl:ObjectProperty';
				$equality_rel = "owl:equivalentProperty";
				$subrel_rel = "rdfs:subPropertyOf";
				break;
			case SMW_NS_ATTRIBUTE:
				$type = 'owl:DatatypeProperty';
				$equality_rel = "owl:equivalentProperty";
				$datatype_rel = "owl:annotationProperty";
				$subatt_rel = "rdfs:subPropertyOf";
				break;
			case NS_CATEGORY:
				$type = 'owl:Class';
				$category_rel = "rdfs:subClassOf";
				$equality_rel = "owl:equivalentClass";
				break;
			default:
				$type = 'smw:Thing';
				$category_rel = "rdf:type";
				$equality_rel = "owl:sameAs";
				break;
		}

		print "\t<" . $type .  " rdf:about=\"" . $pageURIs[0] . "\">\n" .
		      "\t\t<rdfs:label>" . str_replace('&','&amp;',$pageURIs[2]) . "</rdfs:label>\n" .
		      "\t\t<smw:hasArticle rdf:resource=\"&wikiurl;" .
		              $title->getPrefixedURL() . "\"/>\n" .
		      "\t\t<rdfs:isDefinedBy rdf:resource=\"" . 
				      $this->special_url . '/' . $title->getPrefixedURL() . "\"/>\n";

		if ($fullexport) {
			$this->element_queue[$pageURIs[2]] = false; //do not process this element again
			
			// add statements about categories
			if ($category_rel) {
				$dbr =& wfGetDB( DB_MASTER );
				$res = $dbr->query( "SELECT cl_to FROM " . $dbr->tableName('categorylinks') . " WHERE cl_from=$pageID", "SMW::ExportRDF");
				
				if($dbr->numRows( $res ) > 0) {
					$row = $dbr->fetchObject($res);
					while($row) {
						//foreach ($catlinks as $catlink) {
						$catURIs = $this->getURIs($row->cl_to, NS_CATEGORY);

						print "\t\t<" . $category_rel . ' rdf:resource="' . $catURIs[0] .  "\"/>\n";
						
						if (!array_key_exists($catURIs[2], $this->element_queue)) {
							$this->element_queue[$catURIs[2]] = true;
						}
						$row = $dbr->fetchObject($res);
					}
				}
			}
			
			// add statements about equivalence to (external) URIs
			if ($equality_rel) {
				$equalities = SMWGetSpecialProperties($title,SMW_SP_HAS_URI,NULL);
				foreach ($equalities as $equality) {
					print "\t\t<" . $equality_rel . ' rdf:resource="' . $equality[2] . $fragment .  "\"/>\n";
				}
			}
			
			// add rdf:type statements
			if ($datatype_rel) {
				$datatypes = SMWGetSpecialProperties($title,SMW_SP_HAS_TYPE,NULL);
				foreach ($datatypes as $datatype) {
					$th = &SMWGetTypeHandlerByLabel($datatype[2]);
					print "\t\t<$datatype_rel " . 'rdf:resource="&smw;hasType">&smwdt;' . $th->getID() . "</$datatype_rel>\n";
				}
			}
			
			// add rdfs:subPropertyOf statements
			if ($subrel_rel) {
				$relations = SMWGetSpecialProperties($title,SMW_SP_IS_SUBRELATION_OF,NULL);
				foreach ($relations as $relation) {
					$relURIs = $this->getURIs($relation[2],0);
					//@TODO: do type checks here or on saving ...
					print "\t\t<$subrel_rel rdf:resource=\"$relURIs[0]\"/>\n";
					if (!array_key_exists($relURIs[2], $this->element_queue)) {
							$this->element_queue[$relURIs[2]] = true;
					}
				}
			}
			if ($subatt_rel) {
				$attributes = SMWGetSpecialProperties($title,SMW_SP_IS_SUBRELATION_OF,NULL);
				foreach ($attributes as $attribute) {
					$attURIs = $this->getURIs($attribute[2],0);
					//@TODO: do type checks here or on saving ...
					print "\t\t<$subatt_rel rdf:resource=\"$attURIs[0]\"/>\n";
					if (!array_key_exists($attURIs[2], $this->element_queue)) {
							$this->element_queue[$attURIs[2]] = true;
					}
				}
			}
			
			//print all relations and attributes
			$as=SMWGetRelations($title,NULL,NULL);
			if ($as!==false && count($as)>0) {
				foreach($as as $a) {
					$this->processRelation($a);
				}
			}
			$as=SMWGetAttributes($title,NULL,NULL,NULL,NULL);
			if ($as!==false && count($as)>0) {
				foreach($as as $a) {
					$this->processAttribute($a);
				}
			}
		}
		
		print "\t</" . $type . ">\n";
		// Ship data in small chunks (even though browsers often do not display anything
		// before the file is complete -- this might be due to sysntax highlighting features
		// for app/xml). You may want to sleep(1) here for debugging this.
		ob_flush();
		flush();
	}
	
	
	function processRelation($rel) {
		$relURIs = $this->getURIs($rel[1],SMW_NS_RELATION);
		$objURIs = $this->getURIs($rel[3],$rel[2]);
		
		print "\t\t<" . $relURIs[1] . ' rdf:resource="' . $objURIs[0] .  "\"/>\n";
		
		if (!array_key_exists($relURIs[2], $this->element_queue)) {
			$this->element_queue[$relURIs[2]] = true;
		}
		if (!array_key_exists($objURIs[2], $this->element_queue)) {
			$this->element_queue[$objURIs[2]] = true;
		}
		return;
	}
	
	function processAttribute($att) {
		if ( $att[2] != '' ) {
			$attlabel = $att[1] . '#' . $att[2];
		} else {
			$attlabel = $att[1];
		}
		$attURIs = $this->getURIs($attlabel,SMW_NS_ATTRIBUTE);
		
		print "\t\t<" . $attURIs[1] . ' rdf:datatype="' . SMWGetXSDTypeByID($att[3]) .  '">' . $att[4] . '</' . $attURIs[1] . ">\n";
		
		$attname = $attURIs[2];
		if (!array_key_exists($attURIs[2], $this->element_queue)) {
			$this->element_queue[$attURIs[2]] = true;
		}
		return;
	}
	
	
	/** Create full and abbreviated URIs for XML export. The inputs are
	 * $text -- standard text form of the main part of the article name 
	 *          (no namespace, no urlencode)
	 * $namespace -- the namespace constant
	 * 
	 * Returns an array with two entries: the full (0) and the abbreviated
	 * (1) XML URI, and the human readable prefixed text version (2)
	 */
	function getURIs($text, $namespace) {
		global $wgContLang;
		$result = Array();

		$ns_text = $wgContLang->getNsText($namespace);
		if ($ns_text!='') {
			$ns_text .= ':';
			$ns_uri = $this->makeXMLExportId(urlencode(str_replace(' ', '_', $ns_text)));
		} else {
			$ns_uri = '';
		}
		
		$title = Title::newFromText($ns_text . $text);
		if ($title === NULL) {
			print "<!-- Error: \"$nstext$text\" is not a valid article. -->\n";
			return array("","",""); // returning this to an unprepared caller is not elegant, but better than crashing
		}
		
		$text = $title->getText(); //normalize title text
		$fragment = $title->getFragment();
		if( '' != $fragment ) {
			$text .= '#' . $fragment;
		}
		
// 		$uris = SMWGetSpecialProperties($title,SMW_SP_HAS_URI,NULL);
// 		if (count($uris)>0) {
// 			$result[] = $uris[0][3];
// 			$result[] = $uris[0][3];
// 		} else {
			$baseXML = $this->makeXMLExportId(urlencode(str_replace(' ', '_', $text)));
			
			switch ($namespace) {
				case SMW_NS_RELATION:
					$xmlprefix = 'relation:';
					$xmlent = '&relation;';
					break;
				case SMW_NS_ATTRIBUTE:
					$xmlprefix = 'attribute:';
					$xmlent = '&attribute;';
					break;
				default:
					$xmlprefix = 'thing:';
					$xmlent = '&thing;';
					$baseXML = $ns_uri . $baseXML;
					$ns_uri = '';
					break;
			}
			
			//$result[] = $this->wiki_xmlns_xml . '_' . $ns_uri . $baseXML;
			$result[] = $xmlent . $baseXML;
			if (mb_substr($baseXML,0,1)=='-') {
				$result[] = 'wiki:_' . $ns_uri . $baseXML;
			} else {
				$result[] = $xmlprefix . $baseXML;
			}
// 		}
		
		$result[] = $ns_text . $text;
		
		return $result;
	}
	
	/** This function transforms a valid url-encoded URI into a string
	 *  that can be used as an XML-ID. The mapping should be injective.
	 */
	function makeXMLExportId($uri) {
		$uri = str_replace( '-', '-2D', $uri);
		//$uri = str_replace( ':', '-3A', $uri); //already done by PHP
		//$uri = str_replace( '_', '-5F', $uri); //not necessary
		$uri = str_replace( array('"','#','&',"'",'+','%'), 
		                    array('-22','-23','-26','-27','-2B','-'), 
		                    $uri);
		return $uri;
	}
}
?>
